@@ -184,16 +184,13 @@ const static D3D_FEATURE_LEVEL featureLevels[] =
 	D3D_FEATURE_LEVEL_9_3,
 };
 
-void gs_device::InitDevice(const gs_init_data *data, IDXGIAdapter *adapter)
+void gs_device::InitDevice(uint32_t adapterIdx, IDXGIAdapter *adapter)
 {
 	wstring adapterName;
-	DXGI_SWAP_CHAIN_DESC swapDesc;
 	DXGI_ADAPTER_DESC desc;
 	D3D_FEATURE_LEVEL levelUsed = D3D_FEATURE_LEVEL_9_3;
 	HRESULT hr = 0;
 
-	make_swap_desc(swapDesc, data);
-
 	uint32_t createFlags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;
 #ifdef _DEBUG
 	//createFlags |= D3D11_CREATE_DEVICE_DEBUG;
@@ -205,26 +202,19 @@ void gs_device::InitDevice(const gs_init_data *data, IDXGIAdapter *adapter)
 	char *adapterNameUTF8;
 	os_wcs_to_utf8_ptr(adapterName.c_str(), 0, &adapterNameUTF8);
 	blog(LOG_INFO, "Loading up D3D11 on adapter %s (%" PRIu32 ")",
-			adapterNameUTF8, data->adapter);
+			adapterNameUTF8, adapterIdx);
 	bfree(adapterNameUTF8);
 
-	hr = D3D11CreateDeviceAndSwapChain(adapter, D3D_DRIVER_TYPE_UNKNOWN,
+	hr = D3D11CreateDevice(adapter, D3D_DRIVER_TYPE_UNKNOWN,
 			NULL, createFlags, featureLevels,
 			sizeof(featureLevels) / sizeof(D3D_FEATURE_LEVEL),
-			D3D11_SDK_VERSION, &swapDesc,
-			defaultSwap.swap.Assign(), device.Assign(),
+			D3D11_SDK_VERSION, device.Assign(),
 			&levelUsed, context.Assign());
 	if (FAILED(hr))
-		throw UnsupportedHWError("Failed to create device and "
-		                         "swap chain", hr);
+		throw UnsupportedHWError("Failed to create device", hr);
 
 	blog(LOG_INFO, "D3D11 loaded sucessfully, feature level used: %u",
 			(unsigned int)levelUsed);
-
-	defaultSwap.device     = this;
-	defaultSwap.hwnd       = (HWND)data->window.hwnd;
-	defaultSwap.numBuffers = data->num_backbuffers;
-	defaultSwap.Init(data);
 }
 
 static inline void ConvertStencilSide(D3D11_DEPTH_STENCILOP_DESC &desc,
@@ -420,9 +410,8 @@ void gs_device::UpdateViewProjMatrix()
 				&curViewProjMatrix);
 }
 
-gs_device::gs_device(const gs_init_data *data)
-	: curSwapChain         (&defaultSwap),
-	  curToplogy           (D3D11_PRIMITIVE_TOPOLOGY_UNDEFINED)
+gs_device::gs_device(uint32_t adapterIdx)
+	: curToplogy           (D3D11_PRIMITIVE_TOPOLOGY_UNDEFINED)
 {
 	ComPtr<IDXGIAdapter1> adapter;
 
@@ -438,8 +427,8 @@ gs_device::gs_device(const gs_init_data *data)
 	}
 
 	InitCompiler();
-	InitFactory(data->adapter, adapter.Assign());
-	InitDevice(data, adapter);
+	InitFactory(adapterIdx, adapter.Assign());
+	InitDevice(adapterIdx, adapter);
 	device_set_render_target(this, NULL, NULL);
 }
 
@@ -508,14 +497,68 @@ bool device_enum_adapters(
 	}
 }
 
-static bool LogAdapterCallback(void *param, const char *name, uint32_t id)
+static inline void LogAdapterMonitors(IDXGIAdapter1 *adapter)
 {
-	blog(LOG_INFO, "\tAdapter %" PRIu32 ": %s", id, name);
-	UNUSED_PARAMETER(param);
-	return true;
+	UINT i = 0;
+	ComPtr<IDXGIOutput> output;
+
+	while (adapter->EnumOutputs(i++, &output) == S_OK) {
+		DXGI_OUTPUT_DESC desc;
+		if (FAILED(output->GetDesc(&desc)))
+			continue;
+
+		RECT rect = desc.DesktopCoordinates;
+		blog(LOG_INFO, "\t  output %u: "
+				"pos={%d, %d}, "
+				"size={%d, %d}, "
+				"attached=%s",
+				i,
+				rect.left, rect.top,
+				rect.right - rect.left, rect.bottom - rect.top,
+				desc.AttachedToDesktop ? "true" : "false");
+	}
 }
 
-int device_create(gs_device_t **p_device, const gs_init_data *data)
+static inline void LogD3DAdapters()
+{
+	ComPtr<IDXGIFactory1> factory;
+	ComPtr<IDXGIAdapter1> adapter;
+	HRESULT hr;
+	UINT i = 0;
+
+	blog(LOG_INFO, "Available Video Adapters: ");
+
+	IID factoryIID = (GetWinVer() >= 0x602) ? dxgiFactory2 :
+		__uuidof(IDXGIFactory1);
+
+	hr = CreateDXGIFactory1(factoryIID, (void**)factory.Assign());
+	if (FAILED(hr))
+		throw HRError("Failed to create DXGIFactory", hr);
+
+	while (factory->EnumAdapters1(i++, adapter.Assign()) == S_OK) {
+		DXGI_ADAPTER_DESC desc;
+		char name[512] = "";
+
+		hr = adapter->GetDesc(&desc);
+		if (FAILED(hr))
+			continue;
+
+		/* ignore microsoft's 'basic' renderer' */
+		if (desc.VendorId == 0x1414 && desc.DeviceId == 0x8c)
+			continue;
+
+		os_wcs_to_utf8(desc.Description, 0, name, sizeof(name));
+		blog(LOG_INFO, "\tAdapter %u: %s", i, name);
+		blog(LOG_INFO, "\t  Dedicated VRAM: %u",
+				desc.DedicatedVideoMemory);
+		blog(LOG_INFO, "\t  Shared VRAM:    %u",
+				desc.SharedSystemMemory);
+
+		LogAdapterMonitors(adapter);
+	}
+}
+
+int device_create(gs_device_t **p_device, uint32_t adapter)
 {
 	gs_device *device = NULL;
 	int errorcode = GS_SUCCESS;
@@ -523,10 +566,9 @@ int device_create(gs_device_t **p_device, const gs_init_data *data)
 	try {
 		blog(LOG_INFO, "---------------------------------");
 		blog(LOG_INFO, "Initializing D3D11..");
-		blog(LOG_INFO, "Available Video Adapters: ");
-		device_enum_adapters(LogAdapterCallback, nullptr);
+		LogD3DAdapters();
 
-		device = new gs_device(data);
+		device = new gs_device(adapter);
 
 	} catch (UnsupportedHWError error) {
 		blog(LOG_ERROR, "device_create (D3D11): %s (%08lX)", error.str,
@@ -577,6 +619,11 @@ gs_swapchain_t *device_swapchain_create(gs_device_t *device,
 
 void device_resize(gs_device_t *device, uint32_t cx, uint32_t cy)
 {
+	if (!device->curSwapChain) {
+		blog(LOG_WARNING, "device_resize (D3D11): No active swap");
+		return;
+	}
+
 	try {
 		ID3D11RenderTargetView *renderView = NULL;
 		ID3D11DepthStencilView *depthView  = NULL;
@@ -599,18 +646,34 @@ void device_resize(gs_device_t *device, uint32_t cx, uint32_t cy)
 
 void device_get_size(const gs_device_t *device, uint32_t *cx, uint32_t *cy)
 {
-	*cx = device->curSwapChain->target.width;
-	*cy = device->curSwapChain->target.height;
+	if (device->curSwapChain) {
+		*cx = device->curSwapChain->target.width;
+		*cy = device->curSwapChain->target.height;
+	} else {
+		blog(LOG_ERROR, "device_get_size (D3D11): no active swap");
+		*cx = 0;
+		*cy = 0;
+	}
 }
 
 uint32_t device_get_width(const gs_device_t *device)
 {
-	return device->curSwapChain->target.width;
+	if (device->curSwapChain) {
+		return device->curSwapChain->target.width;
+	} else {
+		blog(LOG_ERROR, "device_get_size (D3D11): no active swap");
+		return 0;
+	}
 }
 
 uint32_t device_get_height(const gs_device_t *device)
 {
-	return device->curSwapChain->target.height;
+	if (device->curSwapChain) {
+		return device->curSwapChain->target.height;
+	} else {
+		blog(LOG_ERROR, "device_get_size (D3D11): no active swap");
+		return 0;
+	}
 }
 
 gs_texture_t *device_texture_create(gs_device_t *device, uint32_t width,
@@ -1016,46 +1079,51 @@ gs_zstencil_t *device_get_zstencil_target(const gs_device_t *device)
 void device_set_render_target(gs_device_t *device, gs_texture_t *tex,
 		gs_zstencil_t *zstencil)
 {
-	if (!tex)
-		tex = &device->curSwapChain->target;
-	if (!zstencil)
-		zstencil = &device->curSwapChain->zs;
+	if (device->curSwapChain) {
+		if (!tex)
+			tex = &device->curSwapChain->target;
+		if (!zstencil)
+			zstencil = &device->curSwapChain->zs;
+	}
 
 	if (device->curRenderTarget   == tex &&
 	    device->curZStencilBuffer == zstencil)
 		return;
 
-	if (tex->type != GS_TEXTURE_2D) {
+	if (tex && tex->type != GS_TEXTURE_2D) {
 		blog(LOG_ERROR, "device_set_render_target (D3D11): "
 		                "texture is not a 2D texture");
 		return;
 	}
 
 	gs_texture_2d *tex2d = static_cast<gs_texture_2d*>(tex);
-	if (!tex2d->renderTarget[0]) {
+	if (tex2d && !tex2d->renderTarget[0]) {
 		blog(LOG_ERROR, "device_set_render_target (D3D11): "
 		                "texture is not a render target");
 		return;
 	}
 
-	ID3D11RenderTargetView *rt = tex2d->renderTarget[0];
+	ID3D11RenderTargetView *rt = tex2d ? tex2d->renderTarget[0] : nullptr;
 
 	device->curRenderTarget   = tex2d;
 	device->curRenderSide     = 0;
 	device->curZStencilBuffer = zstencil;
-	device->context->OMSetRenderTargets(1, &rt, zstencil->view);
+	device->context->OMSetRenderTargets(1, &rt,
+			zstencil ? zstencil->view : nullptr);
 }
 
 void device_set_cube_render_target(gs_device_t *device, gs_texture_t *tex,
 		int side, gs_zstencil_t *zstencil)
 {
-	if (!tex) {
-		tex = &device->curSwapChain->target;
-		side = 0;
-	}
+	if (device->curSwapChain) {
+		if (!tex) {
+			tex = &device->curSwapChain->target;
+			side = 0;
+		}
 
-	if (!zstencil)
-		zstencil = &device->curSwapChain->zs;
+		if (!zstencil)
+			zstencil = &device->curSwapChain->zs;
+	}
 
 	if (device->curRenderTarget   == tex  &&
 	    device->curRenderSide     == side &&
@@ -1219,6 +1287,9 @@ void device_draw(gs_device_t *device, enum gs_draw_mode draw_mode,
 		if (!device->curVertexBuffer)
 			throw "No vertex buffer specified";
 
+		if (!device->curSwapChain && !device->curRenderTarget)
+			throw "No render target or swap chain to render to";
+
 		gs_effect_t *effect = gs_get_effect();
 		if (effect)
 			gs_effect_update_params(effect);
@@ -1267,15 +1338,15 @@ void device_load_swapchain(gs_device_t *device, gs_swapchain_t *swapchain)
 {
 	gs_texture_t  *target = device->curRenderTarget;
 	gs_zstencil_t *zs     = device->curZStencilBuffer;
-	bool is_cube = device->curRenderTarget->type == GS_TEXTURE_CUBE;
+	bool is_cube = device->curRenderTarget ?
+		(device->curRenderTarget->type == GS_TEXTURE_CUBE) : false;
 
-	if (target == &device->curSwapChain->target)
-		target = NULL;
-	if (zs == &device->curSwapChain->zs)
-		zs = NULL;
-
-	if (swapchain == NULL)
-		swapchain = &device->defaultSwap;
+	if (device->curSwapChain) {
+		if (target == &device->curSwapChain->target)
+			target = NULL;
+		if (zs == &device->curSwapChain->zs)
+			zs = NULL;
+	}
 
 	device->curSwapChain = swapchain;
 
@@ -1311,7 +1382,11 @@ void device_clear(gs_device_t *device, uint32_t clear_flags,
 
 void device_present(gs_device_t *device)
 {
-	device->curSwapChain->swap->Present(0, 0);
+	if (device->curSwapChain) {
+		device->curSwapChain->swap->Present(0, 0);
+	} else {
+		blog(LOG_WARNING, "device_present (D3D11): No active swap");
+	}
 }
 
 void device_flush(gs_device_t *device)
@@ -1594,12 +1669,8 @@ void device_projection_pop(gs_device_t *device)
 
 void gs_swapchain_destroy(gs_swapchain_t *swapchain)
 {
-	if (!swapchain)
-		return;
-
-	gs_device *device = swapchain->device;
-	if (device->curSwapChain == swapchain)
-		device->curSwapChain = &device->defaultSwap;
+	if (swapchain->device->curSwapChain == swapchain)
+		device_load_swapchain(swapchain->device, nullptr);
 
 	delete swapchain;
 }
@@ -1716,7 +1787,7 @@ uint32_t gs_voltexture_get_height(const gs_texture_t *voltex)
 	return 0;
 }
 
-uint32_t gs_voltexture_getdepth(const gs_texture_t *voltex)
+uint32_t gs_voltexture_get_depth(const gs_texture_t *voltex)
 {
 	/* TODO */
 	UNUSED_PARAMETER(voltex);
