@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013-2014 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2013-2014 by Hugh Bailey <jim@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "util/c99defs.h"
 #include "util/bmem.h"
+#include "util/profiler.h"
 #include "util/text-lookup.h"
 #include "graphics/graphics.h"
 #include "graphics/vec2.h"
@@ -62,11 +63,17 @@ typedef struct obs_module     obs_module_t;
 typedef struct obs_fader      obs_fader_t;
 typedef struct obs_volmeter   obs_volmeter_t;
 
+typedef struct obs_weak_source  obs_weak_source_t;
+typedef struct obs_weak_output  obs_weak_output_t;
+typedef struct obs_weak_encoder obs_weak_encoder_t;
+typedef struct obs_weak_service obs_weak_service_t;
+
 #include "obs-source.h"
 #include "obs-encoder.h"
 #include "obs-output.h"
 #include "obs-service.h"
 #include "obs-audio-controls.h"
+#include "obs-hotkey.h"
 
 /**
  * @file
@@ -105,6 +112,8 @@ enum obs_allow_direct_render {
 };
 
 enum obs_scale_type {
+	OBS_SCALE_DISABLE,
+	OBS_SCALE_POINT,
 	OBS_SCALE_BICUBIC,
 	OBS_SCALE_BILINEAR,
 	OBS_SCALE_LANCZOS
@@ -148,9 +157,6 @@ struct obs_video_info {
 	uint32_t            fps_num;       /**< Output FPS numerator */
 	uint32_t            fps_den;       /**< Output FPS denominator */
 
-	uint32_t            window_width;  /**< Window width */
-	uint32_t            window_height; /**< Window height */
-
 	uint32_t            base_width;    /**< Base compositing width */
 	uint32_t            base_height;   /**< Base compositing height */
 
@@ -161,8 +167,6 @@ struct obs_video_info {
 	/** Video adapter index to use (NOTE: avoid for optimus laptops) */
 	uint32_t            adapter;
 
-	struct gs_window    window;        /**< Window to render to */
-
 	/** Use shaders to convert to different color formats */
 	bool                gpu_conversion;
 
@@ -178,7 +182,6 @@ struct obs_video_info {
 struct obs_audio_info {
 	uint32_t            samples_per_sec;
 	enum speaker_layout speakers;
-	uint64_t            buffer_ms;
 };
 
 /**
@@ -231,6 +234,7 @@ struct obs_source_frame {
 
 	/* used internally by libobs */
 	volatile long       refs;
+	bool                prev_frame;
 };
 
 /* ------------------------------------------------------------------------- */
@@ -239,9 +243,13 @@ struct obs_source_frame {
 /**
  * Initializes OBS
  *
- * @param  locale  The locale to use for modules
+ * @param  locale              The locale to use for modules
+ * @param  module_config_path  Path to module config storage directory
+ *                             (or NULL if none)
+ * @param  store               The profiler name store for OBS to use or NULL
  */
-EXPORT bool obs_startup(const char *locale);
+EXPORT bool obs_startup(const char *locale, const char *module_config_path,
+		profiler_name_store_t *store);
 
 /** Releases all data associated with OBS and terminates the OBS context */
 EXPORT void obs_shutdown(void);
@@ -263,6 +271,13 @@ EXPORT void obs_set_locale(const char *locale);
 /** @return the current locale */
 EXPORT const char *obs_get_locale(void);
 
+/**
+ * Returns the profiler name store (see util/profiler.h) used by OBS, which is
+ * either a name store passed to obs_startup, an internal name store, or NULL
+ * in case obs_initialized() returns false.
+ */
+EXPORT profiler_name_store_t *obs_get_profiler_name_store(void);
+
 /**
  * Sets base video ouput base resolution/fps/format.
  *
@@ -325,6 +340,9 @@ EXPORT int obs_open_module(obs_module_t **module, const char *path,
  */
 EXPORT bool obs_init_module(obs_module_t *module);
 
+/** Logs loaded modules */
+EXPORT void obs_log_loaded_modules(void);
+
 /** Returns the module file name */
 EXPORT const char *obs_get_module_file_name(obs_module_t *module);
 
@@ -389,6 +407,22 @@ EXPORT lookup_t *obs_module_load_locale(obs_module_t *module,
  */
 EXPORT char *obs_find_module_file(obs_module_t *module, const char *file);
 
+/**
+ * Returns the path of a plugin module config file (whether it exists or not)
+ *
+ * @note   Modules should use obs_module_config_path function defined in
+ *         obs-module.h as a more elegant means of getting their files without
+ *         having to specify the module parameter.
+ *
+ * @param  module  The module associated with the path
+ * @param  file    The file to get a path to
+ * @return         Path string, or NULL if not found.  Use bfree to free string.
+ */
+EXPORT char *obs_module_get_config_path(obs_module_t *module, const char *file);
+
+/** Enumerates all source types (inputs, filters, transitions, etc).  */
+EXPORT bool obs_enum_source_types(size_t idx, const char **id);
+
 /**
  * Enumerates all available inputs source types.
  *
@@ -434,16 +468,6 @@ EXPORT audio_t *obs_get_audio(void);
 /** Gets the main video output handler for this OBS context */
 EXPORT video_t *obs_get_video(void);
 
-/**
- * Adds a source to the user source list and increments the reference counter
- * for that source.
- *
- *   The user source list is the list of sources that are accessible by a user.
- * Typically when a transition is active, it is not meant to be accessible by
- * users, so there's no reason for a user to see such a source.
- */
-EXPORT bool obs_add_source(obs_source_t *source);
-
 /** Sets the primary output source for a channel. */
 EXPORT void obs_set_output_source(uint32_t channel, obs_source_t *source);
 
@@ -454,10 +478,13 @@ EXPORT void obs_set_output_source(uint32_t channel, obs_source_t *source);
 EXPORT obs_source_t *obs_get_output_source(uint32_t channel);
 
 /**
- * Enumerates user sources
+ * Enumerates all input sources
  *
  *   Callback function returns true to continue enumeration, or false to end
  * enumeration.
+ *
+ *   Use obs_source_get_ref or obs_source_get_weak_source if you want to retain
+ * a reference after obs_enum_sources finishes
  */
 EXPORT void obs_enum_sources(bool (*enum_proc)(void*, obs_source_t*),
 		void *param);
@@ -491,23 +518,23 @@ EXPORT obs_encoder_t *obs_get_encoder_by_name(const char *name);
 /** Gets an service by its name. */
 EXPORT obs_service_t *obs_get_service_by_name(const char *name);
 
-/** Returns the default effect for generic RGB/YUV drawing */
-EXPORT gs_effect_t *obs_get_default_effect(void);
+enum obs_base_effect {
+	OBS_EFFECT_DEFAULT,            /**< RGB/YUV */
+	OBS_EFFECT_DEFAULT_RECT,       /**< RGB/YUV (using texture_rect) */
+	OBS_EFFECT_OPAQUE,             /**< RGB/YUV (alpha set to 1.0) */
+	OBS_EFFECT_SOLID,              /**< RGB/YUV (solid color only) */
+	OBS_EFFECT_BICUBIC,            /**< Bicubic downscale */
+	OBS_EFFECT_LANCZOS,            /**< Lanczos downscale */
+	OBS_EFFECT_BILINEAR_LOWRES,    /**< Bilinear low resolution downscale */
+	OBS_EFFECT_PREMULTIPLIED_ALPHA,/**< Premultiplied alpha */
+};
 
-/** Returns the default effect for generic RGB/YUV drawing using texture_rect */
-EXPORT gs_effect_t *obs_get_default_rect_effect(void);
+/** Returns a commonly used base effect */
+EXPORT gs_effect_t *obs_get_base_effect(enum obs_base_effect effect);
 
-/** Returns the default effect for generic RGB/YUV drawing (alpha set to 1) */
-EXPORT gs_effect_t *obs_get_opaque_effect(void);
-
-/** Returns the solid effect for drawing solid colors */
-EXPORT gs_effect_t *obs_get_solid_effect(void);
-
-/** Returns the bicubic scaling effect */
-EXPORT gs_effect_t *obs_get_bicubic_effect(void);
-
-/** Returns the lanczos scaling effect */
-EXPORT gs_effect_t *obs_get_lanczos_effect(void);
+/* DEPRECATED: gets texture_rect default effect */
+DEPRECATED_START EXPORT gs_effect_t *obs_get_default_rect_effect(void)
+	DEPRECATED_END;
 
 /** Returns the primary obs signal handler */
 EXPORT signal_handler_t *obs_get_signal_handler(void);
@@ -515,46 +542,46 @@ EXPORT signal_handler_t *obs_get_signal_handler(void);
 /** Returns the primary obs procedure handler */
 EXPORT proc_handler_t *obs_get_proc_handler(void);
 
-/** Adds a draw callback to the main render context */
-EXPORT void obs_add_draw_callback(
-		void (*draw)(void *param, uint32_t cx, uint32_t cy),
-		void *param);
-
-/** Removes a draw callback to the main render context */
-EXPORT void obs_remove_draw_callback(
-		void (*draw)(void *param, uint32_t cx, uint32_t cy),
-		void *param);
-
-/** Changes the size of the main view */
-EXPORT void obs_resize(uint32_t cx, uint32_t cy);
-
 /** Renders the main view */
 EXPORT void obs_render_main_view(void);
 
 /** Sets the master user volume */
 EXPORT void obs_set_master_volume(float volume);
 
-/** Sets the master presentation volume */
-EXPORT void obs_set_present_volume(float volume);
-
 /** Gets the master user volume */
 EXPORT float obs_get_master_volume(void);
 
-/** Gets the master presentation volume */
-EXPORT float obs_get_present_volume(void);
-
 /** Saves a source to settings data */
 EXPORT obs_data_t *obs_save_source(obs_source_t *source);
 
 /** Loads a source from settings data */
 EXPORT obs_source_t *obs_load_source(obs_data_t *data);
 
+typedef void (*obs_load_source_cb)(void *private_data, obs_source_t *source);
+
 /** Loads sources from a data array */
-EXPORT void obs_load_sources(obs_data_array_t *array);
+EXPORT void obs_load_sources(obs_data_array_t *array, obs_load_source_cb cb,
+		void *private_data);
 
 /** Saves sources to a data array */
 EXPORT obs_data_array_t *obs_save_sources(void);
 
+typedef bool (*obs_save_source_filter_cb)(void *data, obs_source_t *source);
+EXPORT obs_data_array_t *obs_save_sources_filtered(obs_save_source_filter_cb cb,
+		void *data);
+
+enum obs_obj_type {
+	OBS_OBJ_TYPE_INVALID,
+	OBS_OBJ_TYPE_SOURCE,
+	OBS_OBJ_TYPE_OUTPUT,
+	OBS_OBJ_TYPE_ENCODER,
+	OBS_OBJ_TYPE_SERVICE
+};
+
+EXPORT enum obs_obj_type obs_obj_get_type(void *obj);
+EXPORT const char *obs_obj_get_id(void *obj);
+EXPORT bool obs_obj_invalid(void *obj);
+
 
 /* ------------------------------------------------------------------------- */
 /* View context */
@@ -581,6 +608,8 @@ EXPORT obs_source_t *obs_view_get_source(obs_view_t *view,
 /** Renders the sources of this view context */
 EXPORT void obs_view_render(obs_view_t *view);
 
+EXPORT uint64_t obs_get_video_frame_time(void);
+
 
 /* ------------------------------------------------------------------------- */
 /* Display context */
@@ -619,13 +648,18 @@ EXPORT void obs_display_remove_draw_callback(obs_display_t *display,
 		void (*draw)(void *param, uint32_t cx, uint32_t cy),
 		void *param);
 
+EXPORT void obs_display_set_enabled(obs_display_t *display, bool enable);
+EXPORT bool obs_display_enabled(obs_display_t *display);
+
+EXPORT void obs_display_set_background_color(obs_display_t *display,
+		uint32_t color);
+
 
 /* ------------------------------------------------------------------------- */
 /* Sources */
 
 /** Returns the translated display name of a source */
-EXPORT const char *obs_source_get_display_name(enum obs_source_type type,
-		const char *id);
+EXPORT const char *obs_source_get_display_name(const char *id);
 
 /**
  * Creates a source of the specified type with the specified settings.
@@ -633,9 +667,16 @@ EXPORT const char *obs_source_get_display_name(enum obs_source_type type,
  *   The "source" context is used for anything related to presenting
  * or modifying video/audio.  Use obs_source_release to release it.
  */
-EXPORT obs_source_t *obs_source_create(enum obs_source_type type,
-		const char *id, const char *name, obs_data_t *settings);
+EXPORT obs_source_t *obs_source_create(const char *id, const char *name,
+		obs_data_t *settings, obs_data_t *hotkey_data);
 
+EXPORT obs_source_t *obs_source_create_private(const char *id,
+		const char *name, obs_data_t *settings);
+
+/* if source has OBS_SOURCE_DO_NOT_DUPLICATE output flag set, only returns a
+ * reference */
+EXPORT obs_source_t *obs_source_duplicate(obs_source_t *source,
+		const char *desired_name, bool create_private);
 /**
  * Adds/releases a reference to a source.  When the last reference is
  * released, the source is destroyed.
@@ -643,6 +684,16 @@ EXPORT obs_source_t *obs_source_create(enum obs_source_type type,
 EXPORT void obs_source_addref(obs_source_t *source);
 EXPORT void obs_source_release(obs_source_t *source);
 
+EXPORT void obs_weak_source_addref(obs_weak_source_t *weak);
+EXPORT void obs_weak_source_release(obs_weak_source_t *weak);
+
+EXPORT obs_source_t *obs_source_get_ref(obs_source_t *source);
+EXPORT obs_weak_source_t *obs_source_get_weak_source(obs_source_t *source);
+EXPORT obs_source_t *obs_weak_source_get_source(obs_weak_source_t *weak);
+
+EXPORT bool obs_weak_source_references_source(obs_weak_source_t *weak,
+		obs_source_t *source);
+
 /** Notifies all references that the source should be released */
 EXPORT void obs_source_remove(obs_source_t *source);
 
@@ -653,16 +704,18 @@ EXPORT bool obs_source_removed(const obs_source_t *source);
 EXPORT uint32_t obs_source_get_output_flags(const obs_source_t *source);
 
 /** Returns capability flags of a source type */
-EXPORT uint32_t obs_get_source_output_flags(enum obs_source_type type,
-		const char *id);
+EXPORT uint32_t obs_get_source_output_flags(const char *id);
 
 /** Gets the default settings for a source type */
-EXPORT obs_data_t *obs_get_source_defaults(enum obs_source_type type,
-		const char *id);
+EXPORT obs_data_t *obs_get_source_defaults(const char *id);
 
 /** Returns the property list, if any.  Free with obs_properties_destroy */
-EXPORT obs_properties_t *obs_get_source_properties(enum obs_source_type type,
-		const char *id);
+EXPORT obs_properties_t *obs_get_source_properties(const char *id);
+
+/** Returns whether the source has custom properties or not */
+EXPORT bool obs_is_source_configurable(const char *id);
+
+EXPORT bool obs_source_configurable(const obs_source_t *source);
 
 /**
  * Returns the properties list for a specific existing source.  Free with
@@ -732,28 +785,22 @@ EXPORT proc_handler_t *obs_source_get_proc_handler(const obs_source_t *source);
 /** Sets the user volume for a source that has audio output */
 EXPORT void obs_source_set_volume(obs_source_t *source, float volume);
 
-/** Sets the presentation volume for a source */
-EXPORT void obs_source_set_present_volume(obs_source_t *source, float volume);
-
 /** Gets the user volume for a source that has audio output */
 EXPORT float obs_source_get_volume(const obs_source_t *source);
 
-/** Gets the presentation volume for a source */
-EXPORT float obs_source_get_present_volume(const obs_source_t *source);
-
 /** Sets the audio sync offset (in nanoseconds) for a source */
 EXPORT void obs_source_set_sync_offset(obs_source_t *source, int64_t offset);
 
 /** Gets the audio sync offset (in nanoseconds) for a source */
 EXPORT int64_t obs_source_get_sync_offset(const obs_source_t *source);
 
-/** Enumerates child sources used by this source */
-EXPORT void obs_source_enum_sources(obs_source_t *source,
+/** Enumerates active child sources used by this source */
+EXPORT void obs_source_enum_active_sources(obs_source_t *source,
 		obs_source_enum_proc_t enum_callback,
 		void *param);
 
-/** Enumerates the entire child source tree used by this source */
-EXPORT void obs_source_enum_tree(obs_source_t *source,
+/** Enumerates the entire active child source tree used by this source */
+EXPORT void obs_source_enum_active_tree(obs_source_t *source,
 		obs_source_enum_proc_t enum_callback,
 		void *param);
 
@@ -765,22 +812,6 @@ EXPORT bool obs_source_active(const obs_source_t *source);
  */
 EXPORT bool obs_source_showing(const obs_source_t *source);
 
-/**
- * Sometimes sources need to be told when to save their settings so they
- * don't have to constantly update and keep track of their settings.  This will
- * call the source's 'save' callback if any, which will save its current
- * data to its settings.
- */
-EXPORT void obs_source_save(obs_source_t *source);
-
-/**
- * Sometimes sources need to be told when they are loading their settings
- * from prior saved data.  This is different from a source 'update' in that
- * it's meant to be used after the source has been created and loaded from
- * somewhere (such as a saved file).
- */
-EXPORT void obs_source_load(obs_source_t *source);
-
 /** Specifies that async video frames should be played as soon as possible */
 #define OBS_SOURCE_FLAG_UNBUFFERED             (1<<0)
 /** Specifies to force audio to mono */
@@ -833,9 +864,59 @@ EXPORT void obs_source_set_enabled(obs_source_t *source, bool enabled);
 EXPORT bool obs_source_muted(const obs_source_t *source);
 EXPORT void obs_source_set_muted(obs_source_t *source, bool muted);
 
+EXPORT bool obs_source_push_to_mute_enabled(obs_source_t *source);
+EXPORT void obs_source_enable_push_to_mute(obs_source_t *source, bool enabled);
+
+EXPORT uint64_t obs_source_get_push_to_mute_delay(obs_source_t *source);
+EXPORT void obs_source_set_push_to_mute_delay(obs_source_t *source,
+		uint64_t delay);
+
+EXPORT bool obs_source_push_to_talk_enabled(obs_source_t *source);
+EXPORT void obs_source_enable_push_to_talk(obs_source_t *source, bool enabled);
+
+EXPORT uint64_t obs_source_get_push_to_talk_delay(obs_source_t *source);
+EXPORT void obs_source_set_push_to_talk_delay(obs_source_t *source,
+		uint64_t delay);
+
+typedef void (*obs_source_audio_capture_t)(void *param, obs_source_t *source,
+		const struct audio_data *audio_data, bool muted);
+
+EXPORT void obs_source_add_audio_capture_callback(obs_source_t *source,
+		obs_source_audio_capture_t callback, void *param);
+EXPORT void obs_source_remove_audio_capture_callback(obs_source_t *source,
+		obs_source_audio_capture_t callback, void *param);
+
+enum obs_deinterlace_mode {
+	OBS_DEINTERLACE_MODE_DISABLE,
+	OBS_DEINTERLACE_MODE_DISCARD,
+	OBS_DEINTERLACE_MODE_RETRO,
+	OBS_DEINTERLACE_MODE_BLEND,
+	OBS_DEINTERLACE_MODE_BLEND_2X,
+	OBS_DEINTERLACE_MODE_LINEAR,
+	OBS_DEINTERLACE_MODE_LINEAR_2X,
+	OBS_DEINTERLACE_MODE_YADIF,
+	OBS_DEINTERLACE_MODE_YADIF_2X
+};
+
+enum obs_deinterlace_field_order {
+	OBS_DEINTERLACE_FIELD_ORDER_TOP,
+	OBS_DEINTERLACE_FIELD_ORDER_BOTTOM
+};
+
+EXPORT void obs_source_set_deinterlace_mode(obs_source_t *source,
+		enum obs_deinterlace_mode mode);
+EXPORT enum obs_deinterlace_mode obs_source_get_deinterlace_mode(
+		const obs_source_t *source);
+EXPORT void obs_source_set_deinterlace_field_order(obs_source_t *source,
+		enum obs_deinterlace_field_order field_order);
+EXPORT enum obs_deinterlace_field_order obs_source_get_deinterlace_field_order(
+		const obs_source_t *source);
+
 /* ------------------------------------------------------------------------- */
 /* Functions used by sources */
 
+EXPORT void *obs_source_get_type_data(obs_source_t *source);
+
 /**
  * Helper function to set the color matrix information when drawing the source.
  *
@@ -892,8 +973,11 @@ EXPORT void obs_source_release_frame(obs_source_t *source,
  *
  * After calling this, set your parameters for the effect, then call
  * obs_source_process_filter_end to draw the filter.
+ *
+ * Returns true if filtering should continue, false if the filter is bypassed
+ * for whatever reason.
  */
-EXPORT void obs_source_process_filter_begin(obs_source_t *filter,
+EXPORT bool obs_source_process_filter_begin(obs_source_t *filter,
 		enum gs_color_format format,
 		enum obs_allow_direct_render allow_direct);
 
@@ -907,24 +991,37 @@ EXPORT void obs_source_process_filter_begin(obs_source_t *filter,
 EXPORT void obs_source_process_filter_end(obs_source_t *filter,
 		gs_effect_t *effect, uint32_t width, uint32_t height);
 
+/**
+ * Draws the filter with a specific technique.
+ *
+ * Before calling this function, first call obs_source_process_filter_begin and
+ * then set the effect parameters, and then call this function to finalize the
+ * filter.
+ */
+EXPORT void obs_source_process_filter_tech_end(obs_source_t *filter,
+		gs_effect_t *effect, uint32_t width, uint32_t height,
+		const char *tech_name);
+
 /** Skips the filter if the filter is invalid and cannot be rendered */
 EXPORT void obs_source_skip_video_filter(obs_source_t *filter);
 
 /**
- * Adds a child source.  Must be called by parent sources on child sources
- * when the child is added.  This ensures that the source is properly activated
- * if the parent is active.
+ * Adds an active child source.  Must be called by parent sources on child
+ * sources when the child is added and active.  This ensures that the source is
+ * properly activated if the parent is active.
  *
  * @returns true if source can be added, false if it causes recursion
  */
-EXPORT bool obs_source_add_child(obs_source_t *parent, obs_source_t *child);
+EXPORT bool obs_source_add_active_child(obs_source_t *parent,
+		obs_source_t *child);
 
 /**
- * Removes a child source.  Must be called by parent sources on child sources
- * when the child is removed.  This ensures that the source is properly
- * deactivated if the parent is active.
+ * Removes an active child source.  Must be called by parent sources on child
+ * sources when the child is removed or inactive.  This ensures that the source
+ * is properly deactivated if the parent is no longer active.
  */
-EXPORT void obs_source_remove_child(obs_source_t *parent, obs_source_t *child);
+EXPORT void obs_source_remove_active_child(obs_source_t *parent,
+		obs_source_t *child);
 
 /** Sends a mouse down/up event to a source */
 EXPORT void obs_source_send_mouse_click(obs_source_t *source,
@@ -956,6 +1053,85 @@ EXPORT uint32_t obs_source_get_base_width(obs_source_t *source);
 /** Gets the base height for a source (not taking in to account filtering) */
 EXPORT uint32_t obs_source_get_base_height(obs_source_t *source);
 
+EXPORT bool obs_source_audio_pending(const obs_source_t *source);
+EXPORT uint64_t obs_source_get_audio_timestamp(const obs_source_t *source);
+EXPORT void obs_source_get_audio_mix(const obs_source_t *source,
+		struct obs_source_audio_mix *audio);
+
+/* ------------------------------------------------------------------------- */
+/* Transition-specific functions */
+enum obs_transition_target {
+	OBS_TRANSITION_SOURCE_A,
+	OBS_TRANSITION_SOURCE_B
+};
+
+EXPORT obs_source_t *obs_transition_get_source(obs_source_t *transition,
+		enum obs_transition_target target);
+EXPORT void obs_transition_clear(obs_source_t *transition);
+
+EXPORT obs_source_t *obs_transition_get_active_source(obs_source_t *transition);
+
+enum obs_transition_mode {
+	OBS_TRANSITION_MODE_AUTO,
+};
+
+EXPORT bool obs_transition_start(obs_source_t *transition,
+		enum obs_transition_mode mode, uint32_t duration_ms,
+		obs_source_t *dest);
+
+EXPORT void obs_transition_set(obs_source_t *transition, obs_source_t *source);
+
+enum obs_transition_scale_type {
+	OBS_TRANSITION_SCALE_MAX_ONLY,
+	OBS_TRANSITION_SCALE_ASPECT,
+	OBS_TRANSITION_SCALE_STRETCH,
+};
+
+EXPORT void obs_transition_set_scale_type(obs_source_t *transition,
+		enum obs_transition_scale_type type);
+EXPORT enum obs_transition_scale_type obs_transition_get_scale_type(
+		const obs_source_t *transition);
+
+EXPORT void obs_transition_set_alignment(obs_source_t *transition,
+		uint32_t alignment);
+EXPORT uint32_t obs_transition_get_alignment(const obs_source_t *transition);
+
+EXPORT void obs_transition_set_size(obs_source_t *transition,
+		uint32_t cx, uint32_t cy);
+EXPORT void obs_transition_get_size(const obs_source_t *transition,
+		uint32_t *cx, uint32_t *cy);
+
+/* function used by transitions */
+
+/**
+ * Enables fixed transitions (videos or specific types of transitions that
+ * are of fixed duration and linearly interpolated
+ */
+EXPORT void obs_transition_enable_fixed(obs_source_t *transition, bool enable,
+		uint32_t duration_ms);
+EXPORT bool obs_transition_fixed(obs_source_t *transition);
+
+typedef void (*obs_transition_video_render_callback_t)(void *data,
+		gs_texture_t *a, gs_texture_t *b, float t,
+		uint32_t cx, uint32_t cy);
+typedef float (*obs_transition_audio_mix_callback_t)(void *data, float t);
+
+EXPORT void obs_transition_video_render(obs_source_t *transition,
+		obs_transition_video_render_callback_t callback);
+
+EXPORT bool obs_transition_audio_render(obs_source_t *transition,
+		uint64_t *ts_out, struct obs_source_audio_mix *audio,
+		uint32_t mixers, size_t channels, size_t sample_rate,
+		obs_transition_audio_mix_callback_t mix_a_callback,
+		obs_transition_audio_mix_callback_t mix_b_callback);
+
+/* swaps transition sources and textures as an optimization and to reduce
+ * memory usage when switching between transitions */
+EXPORT void obs_transition_swap_begin(obs_source_t *tr_dest,
+		obs_source_t *tr_source);
+EXPORT void obs_transition_swap_end(obs_source_t *tr_dest,
+		obs_source_t *tr_source);
+
 
 /* ------------------------------------------------------------------------- */
 /* Scenes */
@@ -968,6 +1144,21 @@ EXPORT uint32_t obs_source_get_base_height(obs_source_t *source);
  */
 EXPORT obs_scene_t *obs_scene_create(const char *name);
 
+EXPORT obs_scene_t *obs_scene_create_private(const char *name);
+
+enum obs_scene_duplicate_type {
+	OBS_SCENE_DUP_REFS,         /**< Source refs only */
+	OBS_SCENE_DUP_COPY,         /**< Fully duplicate */
+	OBS_SCENE_DUP_PRIVATE_REFS, /**< Source refs only (as private) */
+	OBS_SCENE_DUP_PRIVATE_COPY  /**< Fully duplicate (as private) */
+};
+
+/**
+ * Duplicates a scene.
+ */
+EXPORT obs_scene_t *obs_scene_duplicate(obs_scene_t *scene, const char *name,
+		enum obs_scene_duplicate_type type);
+
 EXPORT void        obs_scene_addref(obs_scene_t *scene);
 EXPORT void        obs_scene_release(obs_scene_t *scene);
 
@@ -986,9 +1177,16 @@ EXPORT void obs_scene_enum_items(obs_scene_t *scene,
 		bool (*callback)(obs_scene_t*, obs_sceneitem_t*, void*),
 		void *param);
 
+EXPORT bool obs_scene_reorder_items(obs_scene_t *scene,
+		obs_sceneitem_t * const *item_order, size_t item_order_size);
+
 /** Adds/creates a new scene item for a source */
 EXPORT obs_sceneitem_t *obs_scene_add(obs_scene_t *scene, obs_source_t *source);
 
+typedef void (*obs_scene_atomic_update_func)(void *, obs_scene_t *scene);
+EXPORT void obs_scene_atomic_update(obs_scene_t *scene,
+		obs_scene_atomic_update_func func, void *data);
+
 EXPORT void obs_sceneitem_addref(obs_sceneitem_t *item);
 EXPORT void obs_sceneitem_release(obs_sceneitem_t *item);
 
@@ -1046,7 +1244,27 @@ EXPORT void obs_sceneitem_get_box_transform(const obs_sceneitem_t *item,
 		struct matrix4 *transform);
 
 EXPORT bool obs_sceneitem_visible(const obs_sceneitem_t *item);
-EXPORT void obs_sceneitem_set_visible(obs_sceneitem_t *item, bool visible);
+EXPORT bool obs_sceneitem_set_visible(obs_sceneitem_t *item, bool visible);
+
+struct obs_sceneitem_crop {
+	int left;
+	int top;
+	int right;
+	int bottom;
+};
+
+EXPORT void obs_sceneitem_set_crop(obs_sceneitem_t *item,
+		const struct obs_sceneitem_crop *crop);
+EXPORT void obs_sceneitem_get_crop(const obs_sceneitem_t *item,
+		struct obs_sceneitem_crop *crop);
+
+EXPORT void obs_sceneitem_set_scale_filter(obs_sceneitem_t *item,
+		enum obs_scale_type filter);
+EXPORT enum obs_scale_type obs_sceneitem_get_scale_filter(
+		obs_sceneitem_t *item);
+
+EXPORT void obs_sceneitem_defer_update_begin(obs_sceneitem_t *item);
+EXPORT void obs_sceneitem_defer_update_end(obs_sceneitem_t *item);
 
 
 /* ------------------------------------------------------------------------- */
@@ -1061,8 +1279,24 @@ EXPORT const char *obs_output_get_display_name(const char *id);
  * directshow, or other custom outputs.
  */
 EXPORT obs_output_t *obs_output_create(const char *id, const char *name,
-		obs_data_t *settings);
-EXPORT void obs_output_destroy(obs_output_t *output);
+		obs_data_t *settings, obs_data_t *hotkey_data);
+
+/**
+ * Adds/releases a reference to an output.  When the last reference is
+ * released, the output is destroyed.
+ */
+EXPORT void obs_output_addref(obs_output_t *output);
+EXPORT void obs_output_release(obs_output_t *output);
+
+EXPORT void obs_weak_output_addref(obs_weak_output_t *weak);
+EXPORT void obs_weak_output_release(obs_weak_output_t *weak);
+
+EXPORT obs_output_t *obs_output_get_ref(obs_output_t *output);
+EXPORT obs_weak_output_t *obs_output_get_weak_output(obs_output_t *output);
+EXPORT obs_output_t *obs_weak_output_get_output(obs_weak_output_t *weak);
+
+EXPORT bool obs_weak_output_references_output(obs_weak_output_t *weak,
+		obs_output_t *output);
 
 EXPORT const char *obs_output_get_name(const obs_output_t *output);
 
@@ -1072,6 +1306,32 @@ EXPORT bool obs_output_start(obs_output_t *output);
 /** Stops the output. */
 EXPORT void obs_output_stop(obs_output_t *output);
 
+/**
+ * On reconnection, start where it left of on reconnection.  Note however that
+ * this option will consume extra memory to continually increase delay while
+ * waiting to reconnect.
+ */
+#define OBS_OUTPUT_DELAY_PRESERVE (1<<0)
+
+/**
+ * Sets the current output delay, in seconds (if the output supports delay).
+ *
+ * If delay is currently active, it will set the delay value, but will not
+ * affect the current delay, it will only affect the next time the output is
+ * activated.
+ */
+EXPORT void obs_output_set_delay(obs_output_t *output, uint32_t delay_sec,
+		uint32_t flags);
+
+/** Gets the currently set delay value, in seconds. */
+EXPORT uint32_t obs_output_get_delay(const obs_output_t *output);
+
+/** If delay is active, gets the currently active delay value, in seconds. */
+EXPORT uint32_t obs_output_get_active_delay(const obs_output_t *output);
+
+/** Forces the output to stop.  Usually only used with delay. */
+EXPORT void obs_output_force_stop(obs_output_t *output);
+
 /** Returns whether the output is active */
 EXPORT bool obs_output_active(const obs_output_t *output);
 
@@ -1091,7 +1351,7 @@ EXPORT obs_properties_t *obs_output_properties(const obs_output_t *output);
 EXPORT void obs_output_update(obs_output_t *output, obs_data_t *settings);
 
 /** Specifies whether the output can be paused */
-EXPORT bool obs_output_canpause(const obs_output_t *output);
+EXPORT bool obs_output_can_pause(const obs_output_t *output);
 
 /** Pauses the output (if the functionality is allowed by the output */
 EXPORT void obs_output_pause(obs_output_t *output);
@@ -1196,9 +1456,13 @@ EXPORT uint32_t obs_output_get_width(const obs_output_t *output);
 /** For video outputs, returns the height of the encoded image */
 EXPORT uint32_t obs_output_get_height(const obs_output_t *output);
 
+EXPORT const char *obs_output_get_id(const obs_output_t *output);
+
 /* ------------------------------------------------------------------------- */
 /* Functions used by outputs */
 
+EXPORT void *obs_output_get_type_data(obs_output_t *output);
+
 /** Optionally sets the video conversion info.  Used only for raw output */
 EXPORT void obs_output_set_video_conversion(obs_output_t *output,
 		const struct video_scale_info *conversion);
@@ -1255,7 +1519,7 @@ EXPORT const char *obs_encoder_get_display_name(const char *id);
  * @return           The video encoder context, or NULL if failed or not found.
  */
 EXPORT obs_encoder_t *obs_video_encoder_create(const char *id, const char *name,
-		obs_data_t *settings);
+		obs_data_t *settings, obs_data_t *hotkey_data);
 
 /**
  * Creates an audio encoder context
@@ -1267,10 +1531,25 @@ EXPORT obs_encoder_t *obs_video_encoder_create(const char *id, const char *name,
  * @return           The video encoder context, or NULL if failed or not found.
  */
 EXPORT obs_encoder_t *obs_audio_encoder_create(const char *id, const char *name,
-		obs_data_t *settings, size_t mixer_idx);
+		obs_data_t *settings, size_t mixer_idx,
+		obs_data_t *hotkey_data);
 
-/** Destroys an encoder context */
-EXPORT void obs_encoder_destroy(obs_encoder_t *encoder);
+/**
+ * Adds/releases a reference to an encoder.  When the last reference is
+ * released, the encoder is destroyed.
+ */
+EXPORT void obs_encoder_addref(obs_encoder_t *encoder);
+EXPORT void obs_encoder_release(obs_encoder_t *encoder);
+
+EXPORT void obs_weak_encoder_addref(obs_weak_encoder_t *weak);
+EXPORT void obs_weak_encoder_release(obs_weak_encoder_t *weak);
+
+EXPORT obs_encoder_t *obs_encoder_get_ref(obs_encoder_t *encoder);
+EXPORT obs_weak_encoder_t *obs_encoder_get_weak_encoder(obs_encoder_t *encoder);
+EXPORT obs_encoder_t *obs_weak_encoder_get_encoder(obs_weak_encoder_t *weak);
+
+EXPORT bool obs_weak_encoder_references_encoder(obs_weak_encoder_t *weak,
+		obs_encoder_t *encoder);
 
 EXPORT void obs_encoder_set_name(obs_encoder_t *encoder, const char *name);
 EXPORT const char *obs_encoder_get_name(const obs_encoder_t *encoder);
@@ -1301,6 +1580,22 @@ EXPORT uint32_t obs_encoder_get_width(const obs_encoder_t *encoder);
 /** For video encoders, returns the height of the encoded image */
 EXPORT uint32_t obs_encoder_get_height(const obs_encoder_t *encoder);
 
+/** For audio encoders, returns the sample rate of the audio */
+EXPORT uint32_t obs_encoder_get_sample_rate(const obs_encoder_t *encoder);
+
+/**
+ * Sets the preferred video format for a video encoder.  If the encoder can use
+ * the format specified, it will force a conversion to that format if the
+ * obs output format does not match the preferred format.
+ *
+ * If the format is set to VIDEO_FORMAT_NONE, will revert to the default
+ * functionality of converting only when absolutely necessary.
+ */
+EXPORT void obs_encoder_set_preferred_video_format(obs_encoder_t *encoder,
+		enum video_format format);
+EXPORT enum video_format obs_encoder_get_preferred_video_format(
+		const obs_encoder_t *encoder);
+
 /** Gets the default settings for an encoder type */
 EXPORT obs_data_t *obs_encoder_defaults(const char *id);
 
@@ -1347,6 +1642,12 @@ EXPORT audio_t *obs_encoder_audio(const obs_encoder_t *encoder);
 /** Returns true if encoder is active, false otherwise */
 EXPORT bool obs_encoder_active(const obs_encoder_t *encoder);
 
+EXPORT void *obs_encoder_get_type_data(obs_encoder_t *encoder);
+
+EXPORT const char *obs_encoder_get_id(const obs_encoder_t *encoder);
+
+EXPORT uint32_t obs_get_encoder_caps(const char *encoder_id);
+
 /** Duplicates an encoder packet */
 EXPORT void obs_duplicate_encoder_packet(struct encoder_packet *dst,
 		const struct encoder_packet *src);
@@ -1360,8 +1661,27 @@ EXPORT void obs_free_encoder_packet(struct encoder_packet *packet);
 EXPORT const char *obs_service_get_display_name(const char *id);
 
 EXPORT obs_service_t *obs_service_create(const char *id, const char *name,
-		obs_data_t *settings);
-EXPORT void obs_service_destroy(obs_service_t *service);
+		obs_data_t *settings, obs_data_t *hotkey_data);
+
+EXPORT obs_service_t *obs_service_create_private(const char *id,
+		const char *name, obs_data_t *settings);
+
+/**
+ * Adds/releases a reference to a service.  When the last reference is
+ * released, the service is destroyed.
+ */
+EXPORT void obs_service_addref(obs_service_t *service);
+EXPORT void obs_service_release(obs_service_t *service);
+
+EXPORT void obs_weak_service_addref(obs_weak_service_t *weak);
+EXPORT void obs_weak_service_release(obs_weak_service_t *weak);
+
+EXPORT obs_service_t *obs_service_get_ref(obs_service_t *service);
+EXPORT obs_weak_service_t *obs_service_get_weak_service(obs_service_t *service);
+EXPORT obs_service_t *obs_weak_service_get_service(obs_weak_service_t *weak);
+
+EXPORT bool obs_weak_service_references_service(obs_weak_service_t *weak,
+		obs_service_t *service);
 
 EXPORT const char *obs_service_get_name(const obs_service_t *service);
 
@@ -1408,6 +1728,10 @@ EXPORT void obs_service_apply_encoder_settings(obs_service_t *service,
 		obs_data_t *video_encoder_settings,
 		obs_data_t *audio_encoder_settings);
 
+EXPORT void *obs_service_get_type_data(obs_service_t *service);
+
+EXPORT const char *obs_service_get_id(const obs_service_t *service);
+
 
 /* ------------------------------------------------------------------------- */
 /* Source frame allocation functions */
