@@ -16,10 +16,12 @@
 ******************************************************************************/
 
 #include "window-namedialog.hpp"
+#include "window-basic-main.hpp"
 #include "window-basic-filters.hpp"
 #include "display-helpers.hpp"
 #include "qt-wrappers.hpp"
 #include "visibility-item-widget.hpp"
+#include "item-widget-helpers.hpp"
 #include "obs-app.hpp"
 
 #include <QMessageBox>
@@ -55,6 +57,8 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 	                                "rename",
 	                                OBSBasicFilters::SourceRenamed, this)
 {
+	main = reinterpret_cast<OBSBasic*>(parent);
+
 	ui->setupUi(this);
 	UpdateFilters();
 
@@ -66,8 +70,7 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 	const char *name = obs_source_get_name(source);
 	setWindowTitle(QTStr("Basic.Filters.Title").arg(QT_UTF8(name)));
 
-	connect(ui->preview, SIGNAL(DisplayResized()),
-			this, SLOT(OnPreviewResized()));
+	installEventFilter(CreateShortcutFilter());
 
 	connect(ui->asyncFilters->itemDelegate(),
 			SIGNAL(closeEditor(QWidget*,
@@ -99,32 +102,32 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 
 	if (audioOnly || (audio && !async))
 		ui->asyncLabel->setText(QTStr("Basic.Filters.AudioFilters"));
+
+	auto addDrawCallback = [this] ()
+	{
+		obs_display_add_draw_callback(ui->preview->GetDisplay(),
+				OBSBasicFilters::DrawPreview, this);
+	};
+
+	enum obs_source_type type = obs_source_get_type(source);
+	uint32_t caps = obs_source_get_output_flags(source);
+	bool drawable_type = type == OBS_SOURCE_TYPE_INPUT ||
+		type == OBS_SOURCE_TYPE_SCENE;
+
+	if (drawable_type && (caps & OBS_SOURCE_VIDEO) != 0)
+		connect(ui->preview, &OBSQTDisplay::DisplayCreated,
+				addDrawCallback);
 }
 
 OBSBasicFilters::~OBSBasicFilters()
 {
-	ui->asyncFilters->clear();
-	ui->effectFilters->clear();
-	QApplication::sendPostedEvents(this);
+	ClearListItems(ui->asyncFilters);
+	ClearListItems(ui->effectFilters);
 }
 
 void OBSBasicFilters::Init()
 {
-	gs_init_data init_data = {};
-
 	show();
-
-	QSize previewSize = GetPixelSize(ui->preview);
-	init_data.cx      = uint32_t(previewSize.width());
-	init_data.cy      = uint32_t(previewSize.height());
-	init_data.format  = GS_RGBA;
-	QTToGSWindow(ui->preview->winId(), init_data.window);
-
-	display = obs_display_create(&init_data);
-
-	if (display)
-		obs_display_add_draw_callback(display,
-				OBSBasicFilters::DrawPreview, this);
 }
 
 inline OBSSource OBSBasicFilters::GetFilter(int row, bool async)
@@ -193,10 +196,22 @@ void OBSBasicFilters::RemoveFilter(OBSSource filter)
 		OBSSource curFilter = v.value<OBSSource>();
 
 		if (filter == curFilter) {
-			delete item;
+			DeleteListItem(list, item);
 			break;
 		}
 	}
+
+	const char *filterName = obs_source_get_name(filter);
+	const char *sourceName = obs_source_get_name(source);
+	if (!sourceName || !filterName)
+		return;
+
+	const char *filterId = obs_source_get_id(filter);
+
+	blog(LOG_INFO, "User removed filter '%s' (%s) from source '%s'",
+			filterName, filterId, sourceName);
+
+	main->SaveProject();
 }
 
 struct FilterOrderInfo {
@@ -221,7 +236,7 @@ void OBSBasicFilters::ReorderFilter(QListWidget *list,
 			if ((int)idx != i) {
 				bool sel = (list->currentRow() == i);
 
-				listItem = list->takeItem(i);
+				listItem = TakeListItem(list, i);
 				if (listItem)  {
 					list->insertItem((int)idx, listItem);
 					SetupVisibilityItem(list,
@@ -269,8 +284,8 @@ void OBSBasicFilters::UpdateFilters()
 	if (!source)
 		return;
 
-	ui->effectFilters->clear();
-	ui->asyncFilters->clear();
+	ClearListItems(ui->effectFilters);
+	ClearListItems(ui->asyncFilters);
 
 	obs_source_enum_filters(source,
 			[] (obs_source_t*, obs_source_t *filter, void *p)
@@ -280,6 +295,8 @@ void OBSBasicFilters::UpdateFilters()
 
 				window->AddFilter(filter);
 			}, this);
+
+	main->SaveProject();
 }
 
 static bool filter_compatible(bool async, uint32_t sourceFlags,
@@ -308,10 +325,8 @@ QMenu *OBSBasicFilters::CreateAddFilterPopupMenu(bool async)
 
 	QMenu *popup = new QMenu(QTStr("Add"), this);
 	while (obs_enum_filter_types(idx++, &type)) {
-		const char *name = obs_source_get_display_name(
-				OBS_SOURCE_TYPE_FILTER, type);
-		uint32_t filterFlags = obs_get_source_output_flags(
-				OBS_SOURCE_TYPE_FILTER, type);
+		const char *name = obs_source_get_display_name(type);
+		uint32_t filterFlags = obs_get_source_output_flags(type);
 
 		if (!filter_compatible(async, sourceFlags, filterFlags))
 			continue;
@@ -337,8 +352,7 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 {
 	if (id && *id) {
 		obs_source_t *existing_filter;
-		string name = obs_source_get_display_name(
-				OBS_SOURCE_TYPE_FILTER, id);
+		string name = obs_source_get_display_name(id);
 
 		bool success = NameDialog::AskForName(this,
 				QTStr("Basic.Filters.AddFilter.Title"),
@@ -366,9 +380,15 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 			return;
 		}
 
-		obs_source_t *filter = obs_source_create(OBS_SOURCE_TYPE_FILTER,
-				id, name.c_str(), nullptr);
+		obs_source_t *filter = obs_source_create(id, name.c_str(),
+				nullptr, nullptr);
 		if (filter) {
+			const char *sourceName = obs_source_get_name(source);
+
+			blog(LOG_INFO, "User added filter '%s' (%s) "
+					"to source '%s'",
+					name.c_str(), id, sourceName);
+
 			obs_source_filter_add(source, filter);
 			obs_source_release(filter);
 		}
@@ -384,35 +404,16 @@ void OBSBasicFilters::AddFilterFromAction()
 	AddNewFilter(QT_TO_UTF8(action->data().toString()));
 }
 
-void OBSBasicFilters::OnPreviewResized()
-{
-	if (resizeTimer)
-		killTimer(resizeTimer);
-	resizeTimer = startTimer(100);
-}
-
 void OBSBasicFilters::closeEvent(QCloseEvent *event)
 {
 	QDialog::closeEvent(event);
 	if (!event->isAccepted())
 		return;
 
-	// remove draw callback and release display in case our drawable
-	// surfaces go away before the destructor gets called
-	obs_display_remove_draw_callback(display,
-			OBSBasicFilters::DrawPreview, this);
-	display = nullptr;
-}
+	obs_display_remove_draw_callback (ui->preview->GetDisplay(),
+		OBSBasicFilters::DrawPreview, this);
 
-void OBSBasicFilters::timerEvent(QTimerEvent *event)
-{
-	if (event->timerId() == resizeTimer) {
-		killTimer(resizeTimer);
-		resizeTimer = 0;
-
-		QSize size = GetPixelSize(ui->preview);
-		obs_display_resize(display, size.width(), size.height());
-	}
+	main->SaveProject();
 }
 
 /* OBS Signals */
@@ -685,6 +686,11 @@ void OBSBasicFilters::FilterNameEdited(QWidget *editor, QListWidget *list)
 				QTStr("NoNameEntered.Text"));
 		}
 	} else {
+		const char *sourceName = obs_source_get_name(source);
+
+		blog(LOG_INFO, "User renamed filter '%s' on source '%s' to '%s'",
+				prevName, sourceName, name.c_str());
+
 		listItem->setText(QT_UTF8(name.c_str()));
 		obs_source_set_name(filter, name.c_str());
 	}
