@@ -1,4 +1,5 @@
 #include <string>
+#include <algorithm>
 #include <QMessageBox>
 #include "audio-encoders.hpp"
 #include "window-basic-main.hpp"
@@ -27,10 +28,10 @@ static void OBSStreamStopping(void *data, calldata_t *params)
 
 	int sec = (int)obs_output_get_active_delay(obj);
 	if (sec == 0)
-		return;
-
-	QMetaObject::invokeMethod(output->main,
-			"StreamDelayStopping", Q_ARG(int, sec));
+		QMetaObject::invokeMethod(output->main, "StreamStopping");
+	else
+		QMetaObject::invokeMethod(output->main,
+				"StreamDelayStopping", Q_ARG(int, sec));
 }
 
 static void OBSStartStreaming(void *data, calldata_t *params)
@@ -75,6 +76,14 @@ static void OBSStopRecording(void *data, calldata_t *params)
 	UNUSED_PARAMETER(params);
 }
 
+static void OBSRecordStopping(void *data, calldata_t *params)
+{
+	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
+	QMetaObject::invokeMethod(output->main, "RecordStopping");
+
+	UNUSED_PARAMETER(params);
+}
+
 static void FindBestFilename(string &strPath, bool noSpace)
 {
 	int num = 2;
@@ -281,7 +290,7 @@ SimpleOutput::SimpleOutput(OBSBasic *main_) : BasicOutputHandler(main_)
 
 	streamDelayStarting.Connect(obs_output_get_signal_handler(streamOutput),
 			"starting", OBSStreamStarting, this);
-	streamDelayStopping.Connect(obs_output_get_signal_handler(streamOutput),
+	streamStopping.Connect(obs_output_get_signal_handler(streamOutput),
 			"stopping", OBSStreamStopping, this);
 
 	startStreaming.Connect(obs_output_get_signal_handler(streamOutput),
@@ -304,6 +313,8 @@ SimpleOutput::SimpleOutput(OBSBasic *main_) : BasicOutputHandler(main_)
 			"start", OBSStartRecording, this);
 	stopRecording.Connect(obs_output_get_signal_handler(fileOutput),
 			"stop", OBSStopRecording, this);
+	recordStopping.Connect(obs_output_get_signal_handler(fileOutput),
+			"stopping", OBSRecordStopping, this);
 }
 
 int SimpleOutput::GetAudioBitrate() const
@@ -342,6 +353,7 @@ void SimpleOutput::Update()
 
 	preset = config_get_string(main->Config(), "SimpleOutput", presetType);
 
+	obs_data_set_string(h264Settings, "rate_control", "CBR");
 	obs_data_set_int(h264Settings, "bitrate", videoBitrate);
 
 	if (advanced) {
@@ -528,6 +540,14 @@ bool SimpleOutput::StartStreaming(obs_service_t *service)
 			"DelaySec");
 	bool preserveDelay = config_get_bool(main->Config(), "Output",
 			"DelayPreserve");
+	const char *bindIP = config_get_string(main->Config(), "Output",
+			"BindIP");
+
+	obs_data_t *settings = obs_data_create();
+	obs_data_set_string(settings, "bind_ip", bindIP);
+	obs_output_update(streamOutput, settings);
+	obs_data_release(settings);
+
 	if (!reconnect)
 		maxRetries = 0;
 
@@ -544,6 +564,18 @@ bool SimpleOutput::StartStreaming(obs_service_t *service)
 	return false;
 }
 
+static void ensure_directory_exists(string &path)
+{
+	replace(path.begin(), path.end(), '\\', '/');
+
+	size_t last = path.rfind('/');
+	if (last == string::npos)
+		return;
+
+	string directory = path.substr(0, last);
+	os_mkdirs(directory.c_str());
+}
+
 bool SimpleOutput::StartRecording()
 {
 	if (usingRecordingPreset) {
@@ -589,11 +621,10 @@ bool SimpleOutput::StartRecording()
 
 	strPath += GenerateSpecifiedFilename(ffmpegOutput ? "avi" : format,
 			noSpace, filenameFormat);
+	ensure_directory_exists(strPath);
 	if (!overwriteIfExists)
 		FindBestFilename(strPath, noSpace);
 
-	SetupOutputs();
-
 	if (!ffmpegOutput) {
 		obs_output_set_video_encoder(fileOutput, h264Recording);
 		obs_output_set_audio_encoder(fileOutput, aacRecording, 0);
@@ -761,7 +792,7 @@ AdvancedOutput::AdvancedOutput(OBSBasic *main_) : BasicOutputHandler(main_)
 
 	streamDelayStarting.Connect(obs_output_get_signal_handler(streamOutput),
 			"starting", OBSStreamStarting, this);
-	streamDelayStopping.Connect(obs_output_get_signal_handler(streamOutput),
+	streamStopping.Connect(obs_output_get_signal_handler(streamOutput),
 			"stopping", OBSStreamStopping, this);
 
 	startStreaming.Connect(obs_output_get_signal_handler(streamOutput),
@@ -773,6 +804,8 @@ AdvancedOutput::AdvancedOutput(OBSBasic *main_) : BasicOutputHandler(main_)
 			"start", OBSStartRecording, this);
 	stopRecording.Connect(obs_output_get_signal_handler(fileOutput),
 			"stop", OBSStopRecording, this);
+	recordStopping.Connect(obs_output_get_signal_handler(fileOutput),
+			"stopping", OBSRecordStopping, this);
 }
 
 void AdvancedOutput::UpdateStreamSettings()
@@ -1052,6 +1085,14 @@ bool AdvancedOutput::StartStreaming(obs_service_t *service)
 			"DelaySec");
 	bool preserveDelay = config_get_bool(main->Config(), "Output",
 			"DelayPreserve");
+	const char *bindIP = config_get_string(main->Config(), "Output",
+			"BindIP");
+
+	obs_data_t *settings = obs_data_create();
+	obs_data_set_string(settings, "bind_ip", bindIP);
+	obs_output_update(streamOutput, settings);
+	obs_data_release(settings);
+
 	if (!reconnect)
 		maxRetries = 0;
 
@@ -1123,6 +1164,7 @@ bool AdvancedOutput::StartRecording()
 
 		strPath += GenerateSpecifiedFilename(recFormat, noSpace,
 							filenameFormat);
+		ensure_directory_exists(strPath);
 		if (!overwriteIfExists)
 			FindBestFilename(strPath, noSpace);
 
