@@ -9,6 +9,7 @@ protected:
 	DeckLink                *decklink = nullptr;
 	DeckLinkDevice          *device = nullptr;
 	DeckLinkDeviceMode      *mode = nullptr;
+	BMDPixelFormat          pixelFormat = bmdFormat8BitYUV;
 	ComPtr<IDeckLinkInput>  input;
 	volatile long           refCount = 1;
 
@@ -19,6 +20,7 @@ protected:
 
 public:
 	DeckLinkDeviceInstance(DeckLink *decklink, DeckLinkDevice *device);
+	virtual ~DeckLinkDeviceInstance();
 
 	inline DeckLinkDevice *GetDevice() const {return device;}
 	inline long long GetActiveModeId() const
@@ -26,6 +28,8 @@ public:
 		return mode ? mode->GetId() : 0;
 	}
 
+	inline BMDPixelFormat GetActivePixelFormat() const {return pixelFormat;}
+
 	inline DeckLinkDeviceMode *GetMode() const {return mode;}
 
 	bool StartCapture(DeckLinkDeviceMode *mode);
